/// <reference types="wechat-miniprogram" />
import { AUDIO_STATUS } from './constants';
import { AudioManagerEvents, AudioManagerInfo } from './types';
import { isValidProgress, logAm, formatTimeString } from './utils';
export declare class AudioManager {
    /**
     * 初始化 AudioManager，为其注册生命周期
     */
    static init(): void;
    /**
     * 获取全局音频播放设置
     * @returns audioManager 当前 audioManager 实例
     */
    static get(): {
        paused: boolean;
        currentTime: number;
        duration: number;
        buffered: number;
        status: number;
        coverImgUrl?: string | undefined;
        epname?: string | undefined;
        playbackRate?: number | undefined;
        protocol?: string | undefined;
        singer?: string | undefined;
        src?: string | undefined;
        startTime?: number | undefined;
        title?: string | undefined;
        webUrl?: string | undefined;
        onCanplay?: ((callback: WechatMiniprogram.OnCanplayCallback) => void) | undefined;
        onEnded?: ((callback: WechatMiniprogram.OnEndedCallback) => void) | undefined;
        onError?: ((callback: WechatMiniprogram.BackgroundAudioManagerOnErrorCallback) => void) | undefined;
        onNext?: ((callback: WechatMiniprogram.OnNextCallback) => void) | undefined;
        onPause?: ((callback: WechatMiniprogram.OnPauseCallback) => void) | undefined;
        onPlay?: ((callback: WechatMiniprogram.OnPlayCallback) => void) | undefined;
        onPrev?: ((callback: WechatMiniprogram.OnPrevCallback) => void) | undefined;
        onSeeked?: ((callback: WechatMiniprogram.OnSeekedCallback) => void) | undefined;
        onSeeking?: ((callback: WechatMiniprogram.OnSeekingCallback) => void) | undefined;
        onStop?: ((callback: WechatMiniprogram.InnerAudioContextOnStopCallback) => void) | undefined;
        onTimeUpdate?: ((callback: WechatMiniprogram.OnTimeUpdateCallback) => void) | undefined;
        onWaiting?: ((callback: WechatMiniprogram.OnWaitingCallback) => void) | undefined;
        pause?: (() => void) | undefined;
        play?: (() => void) | undefined;
        seek?: ((currentTime: number) => void) | undefined;
        stop?: (() => void) | undefined;
    };
    /**
     * 设置全局音频播放设置
     */
    static set(nextManager: Partial<AudioManagerInfo>): {
        hasSrcChanged: boolean;
        paused: boolean;
        currentTime: number;
        duration: number;
        buffered: number;
        status: number;
        coverImgUrl?: string | undefined;
        epname?: string | undefined;
        playbackRate?: number | undefined;
        protocol?: string | undefined;
        singer?: string | undefined;
        src?: string | undefined;
        startTime?: number | undefined;
        title?: string | undefined;
        webUrl?: string | undefined;
        onCanplay?: ((callback: WechatMiniprogram.OnCanplayCallback) => void) | undefined;
        onEnded?: ((callback: WechatMiniprogram.OnEndedCallback) => void) | undefined;
        onError?: ((callback: WechatMiniprogram.BackgroundAudioManagerOnErrorCallback) => void) | undefined;
        onNext?: ((callback: WechatMiniprogram.OnNextCallback) => void) | undefined;
        onPause?: ((callback: WechatMiniprogram.OnPauseCallback) => void) | undefined;
        onPlay?: ((callback: WechatMiniprogram.OnPlayCallback) => void) | undefined;
        onPrev?: ((callback: WechatMiniprogram.OnPrevCallback) => void) | undefined;
        onSeeked?: ((callback: WechatMiniprogram.OnSeekedCallback) => void) | undefined;
        onSeeking?: ((callback: WechatMiniprogram.OnSeekingCallback) => void) | undefined;
        onStop?: ((callback: WechatMiniprogram.InnerAudioContextOnStopCallback) => void) | undefined;
        onTimeUpdate?: ((callback: WechatMiniprogram.OnTimeUpdateCallback) => void) | undefined;
        onWaiting?: ((callback: WechatMiniprogram.OnWaitingCallback) => void) | undefined;
        pause?: (() => void) | undefined;
        play?: (() => void) | undefined;
        seek?: ((currentTime: number) => void) | undefined;
        stop?: (() => void) | undefined;
    } | undefined;
    /**
     * 播放背景音乐，同时需要定义若干播放条件，src 必传
     */
    static play(am: Partial<AudioManagerInfo>): void;
    /**
     * 播放背景音乐，只需传入 src
     */
    static playSrc(src: string): void;
    /**
     * 使指定 am 跳到指定的地方开始播放或继续播放
     * @param {Object} am audioManager，必须传入 startTime 和 src
     */
    static seek(am: AudioManagerInfo): void;
    /**
     * 暂停播放
     */
    static pause(): void;
    /**
     * 停止播放，会丢失 src 等信息
     */
    static stop(): void;
    /**
     * 在一段时间后，停止播放
     * @param {number} countdown 倒计时持续时间
     */
    static stopLater(countdown: number): void;
    static isPlaying(am?: AudioManagerInfo): boolean;
    /**
     * 向 audioManager 订阅事件，具体可监听的事件可见 EVENT_NAMES
     * @param {object} callbacks 一组事件名，回调的映射
     * @returns {function} 一个批量取消监听的方法
     */
    static on(callbacks: AudioManagerEvents): () => void;
}
export { AUDIO_STATUS, isValidProgress, logAm, formatTimeString };
export type { AudioManagerInfo };
